-- Required due to HTML strings with ampersands in them
set define off  

CREATE OR REPLACE PACKAGE BODY PORTAL_WF_REQ_AP_NOTIFICATION AS
/* $Header: PORREQAPWFB.pls 115.1 2002/4/25 08:00:00 pkm ship        $ */ 
/***********************************************************************/
/* JPeters, JRPJR, Inc.                                                */
/* john.peters@jrpjr.com                                               */
/***********************************************************************/
/* Revision History                                                    */
/* 04/25/02  JPeters   Initial Creation                                */
/* 05/13/02  JPeters   - Added logic to not print headers when         */
/*                       no attachments exist                          */
/* 05/13/02  JPeters   - Added a Supplier Block to the Nofification    */
/* 05/17/02  JPeters   - Put in distinct for supplier listing          */
/*                                                                     */
/*                                                                     */
/***********************************************************************/

TYPE supplier_record IS RECORD (
  suggested_vendor_name        po_requisition_lines.SUGGESTED_VENDOR_NAME%TYPE);

TYPE dist_record IS RECORD (
  line_num         po_requisition_lines.line_num%TYPE,
  dist_num         po_req_distributions_all.distribution_num%TYPE,
  dist_acct        varchar2(200),
  allocation_type  po_req_distributions_all.allocation_type%TYPE,
  allocation_value po_req_distributions_all.allocation_value%TYPE);

TYPE attachment_record IS RECORD (
  seq_num          fnd_attached_documents.seq_num%TYPE,
  attention_to     fnd_document_categories_tl.user_name%TYPE,
  doc_type         fnd_document_datatypes.user_name%TYPE,
  description      fnd_documents_tl.description%TYPE,
  attachment_txt   varchar2(4000),
  attachment_html  varchar2(4000));


-- SetStartupValues
--  Iinitialize/assigns startup values to workflow attributes.
--  These are used to display the requisition distributions in
--  the Req Approval Email.
--
-- IN
--   itemtype, itemkey, actid, funcmode
-- OUT
--   Resultout
--    - Completed   - Activity was completed without any errors.
--

procedure Set_Startup_Values(itemtype        in varchar2,
                             itemkey         in varchar2,
                             actid           in number,
                             funcmode        in varchar2,
                             resultout       out varchar2) is


BEGIN

  -- Do nothing in cancel or timeout mode
  --
  if (funcmode <> wf_engine.eng_run) then

      resultout := wf_engine.eng_null;
      return;

  end if;

  wf_engine.SetItemAttrText(itemtype => itemtype,
                            itemkey  => itemkey,
                            aname    => 'PORTAL_REQ_SUPPLIERS',
                            avalue   => 'PLSQL:PORTAL_WF_REQ_AP_NOTIFICATION.GET_REQ_SUPPLIER_DETAILS/'||
                                        itemtype||':'||
                                        itemkey);
                                        
  wf_engine.SetItemAttrText(itemtype => itemtype,
                            itemkey  => itemkey,
                            aname    => 'PORTAL_REQ_DISTRIBUTIONS',
                            avalue   => 'PLSQL:PORTAL_WF_REQ_AP_NOTIFICATION.GET_REQ_DISTR_DETAILS/'||
                                        itemtype||':'||
                                        itemkey);

  wf_engine.SetItemAttrText(itemtype => itemtype,
                            itemkey  => itemkey,
                            aname    => 'PORTAL_REQ_ATTACHMENTS',
                            avalue   => 'PLSQL:PORTAL_WF_REQ_AP_NOTIFICATION.GET_REQ_ATTACHMENTS/'||
                                        itemtype||':'||
                                        itemkey);
  --
  resultout := wf_engine.eng_completed || ':' ||  'ACTIVITY_PERFORMED';
  --

END Set_Startup_Values;



FUNCTION get_long_text_attachment(p_media_id in number)
  RETURN  VARCHAR2
IS
  v_long_string varchar2(32767);
  v_attachment_string varchar2(4000);

BEGIN
  v_attachment_string := NULL;
  select fdlt.LONG_TEXT
    into v_long_string
    from FND_DOCUMENTS_LONG_TEXT fdlt 
   where fdlt.MEDIA_ID = p_media_id 
   ;
  v_attachment_string := substr(v_long_string,1,4000); 
   
  return v_attachment_string;
  
END get_long_text_attachment;


FUNCTION get_file_attachment(p_media_id in number)
  RETURN  VARCHAR2
IS
  v_attachment_string varchar2(4000);

BEGIN
  v_attachment_string := NULL;
  select FILE_NAME
    into v_attachment_string
    from FND_LOBS
   where FILE_ID = p_media_id
   ; 
   
  return v_attachment_string;
  
END get_file_attachment;


FUNCTION get_site_profile_option(p_profile_option in varchar2)
  RETURN  VARCHAR2
IS
  v_site_value varchar2(240);
BEGIN
  v_site_value := 'Profile Option: ' || p_profile_option || ' not set at site level';
  select fpov.PROFILE_OPTION_VALUE
    into v_site_value
    from FND_PROFILE_OPTION_VALUES fpov,
         FND_PROFILE_OPTIONS fpo
   where fpov.LEVEL_ID = 10001  -- site 
     and fpov.LEVEL_VALUE = 0   -- site 
     and fpo.PROFILE_OPTION_ID = fpov.PROFILE_OPTION_ID
     and fpo.PROFILE_OPTION_NAME = p_profile_option
  ;    
   
  return v_site_value;
  
END get_site_profile_option;



PROCEDURE get_req_supplier_details(document_id     in      varchar2,
                                   display_type    in      varchar2,
                                   document        in out  varchar2,
                                   document_type   in out  varchar2) IS

-- -------------------------------------------------------------------------
--  This procedure builds an HTML table containing a requisitions supplier info 
--  for display in the Workflow Notification Message.
--
-- --------------------------------------------------------------------------
  l_item_type        wf_items.item_type%TYPE;
  l_item_key         wf_items.item_key%TYPE;
  v_do_header        varchar2(1) := 'Y';
  l_line             supplier_record;
  l_document         VARCHAR2(32000) := '';
  l_document_id      po_requisition_lines.requisition_header_id%TYPE;
  l_org_id           po_requisition_lines.org_id%TYPE;
  NL                 VARCHAR2(1) := fnd_global.newline;

CURSOR supplier_csr(v_document_id NUMBER) IS
SELECT distinct SUGGESTED_VENDOR_NAME
  FROM PO_REQUISITION_LINES rql
 WHERE rql.REQUISITION_HEADER_ID = v_document_id
   AND rql.SUGGESTED_VENDOR_NAME is not NULL
   AND NVL(rql.CANCEL_FLAG,'N') = 'N'
   AND NVL(rql.MODIFIED_BY_AGENT_FLAG, 'N') = 'N'
 ;

BEGIN
  l_item_type := substr(document_id, 1, instr(document_id, ':') - 1);
  l_item_key := substr(document_id, instr(document_id, ':') + 1,
                       length(document_id) - 2);

  l_document_id := wf_engine.GetItemAttrNumber
                                        (itemtype   => l_item_type,
                                         itemkey    => l_item_key,
                                         aname      => 'DOCUMENT_ID');

  l_org_id := wf_engine.GetItemAttrNumber
                                        (itemtype   => l_item_type,
                                         itemkey    => l_item_key,
                                         aname      => 'ORG_ID');

  fnd_client_info.set_org_context(to_char(l_org_id));

  if (display_type = 'text/html') then
    
    open supplier_csr(l_document_id);

    loop

      fetch supplier_csr into l_line;

      exit when supplier_csr%notfound;
      
      if (v_do_header = 'Y') then
        v_do_header := 'N';
        l_document := NL || NL || '<!-- REQ_SUPPLIER_DETAILS -->'|| NL || NL || '<P><U>';
        l_document := l_document || 'Suggested Suppliers';
        l_document := l_document || '</U>';
        l_document := l_document || '<TABLE border=0 cellpadding=2 cellspacing=1>';
      end if;

      l_document := l_document || '<TR>' || NL;
      l_document := l_document || '<TD valign=top nowrap><B>' || nvl(l_line.suggested_vendor_name, '&nbsp') || '</B></TD>' || NL;
      l_document := l_document || '</TR>' || NL;

    end loop;

    close supplier_csr;

    if (v_do_header = 'N') then
      l_document := l_document || '</TABLE></P>' || NL;
    end if;  

  else


    open supplier_csr(l_document_id);

    loop

      fetch supplier_csr into l_line;

      exit when supplier_csr%notfound;

      if (v_do_header = 'Y') then
        v_do_header := 'N';
        l_document := l_document || 'Requisition Suggested Suppliers' || NL || NL;
      end if;
      
      l_document := l_document || ' Suggested Supplier:' || l_line.suggested_vendor_name || NL;

      end loop;

      close supplier_csr;

  end if;

  document := l_document;

END get_req_supplier_details;



PROCEDURE get_req_distr_details(document_id     in      varchar2,
                                display_type    in      varchar2,
                                document        in out  varchar2,
                                document_type   in out  varchar2) IS

-- -------------------------------------------------------------------------
--  This procedure builds an HTML table containing a requisitions distribution
--  lines for display in the Workflow Notification Message.
--
-- --------------------------------------------------------------------------


  l_item_type        wf_items.item_type%TYPE;
  l_item_key         wf_items.item_key%TYPE;
  l_document_id      po_requisition_lines.requisition_header_id%TYPE;
  l_org_id           po_requisition_lines.org_id%TYPE;
  l_line             dist_record;
  l_document         VARCHAR2(32000) := '';
  l_currency_code    fnd_currencies.currency_code%TYPE;
  NL                 VARCHAR2(1) := fnd_global.newline;


CURSOR dist_csr(v_document_id NUMBER) IS
SELECT rql.LINE_NUM,
       rqd.DISTRIBUTION_NUM,
       gcc.SEGMENT1 || '.' ||
       gcc.SEGMENT2 || '.' ||
       gcc.SEGMENT3 || '.' ||
       gcc.SEGMENT4 || '.' ||
       gcc.SEGMENT5 || '.' ||
       gcc.SEGMENT6 || '.' ||
       gcc.SEGMENT7 || '.' ||
       gcc.SEGMENT8 dist_acct,
       rqd.ALLOCATION_TYPE,
       rqd.ALLOCATION_VALUE
  FROM GL_CODE_COMBINATIONS gcc,
       PO_REQ_DISTRIBUTIONS rqd,
       PO_REQUISITION_LINES rql
 WHERE rqd.CODE_COMBINATION_ID = gcc.CODE_COMBINATION_ID
   AND rql.REQUISITION_LINE_ID = rqd.REQUISITION_LINE_ID
   AND rql.REQUISITION_HEADER_ID = v_document_id
   AND NVL(rql.CANCEL_FLAG,'N') = 'N'
   AND NVL(rql.MODIFIED_BY_AGENT_FLAG, 'N') = 'N'
 ORDER BY rql.LINE_NUM, rqd.DISTRIBUTION_NUM;

BEGIN

  l_item_type := substr(document_id, 1, instr(document_id, ':') - 1);
  l_item_key := substr(document_id, instr(document_id, ':') + 1,
                       length(document_id) - 2);

  l_document_id := wf_engine.GetItemAttrNumber
                                        (itemtype   => l_item_type,
                                         itemkey    => l_item_key,
                                         aname      => 'DOCUMENT_ID');

  l_org_id := wf_engine.GetItemAttrNumber
                                        (itemtype   => l_item_type,
                                         itemkey    => l_item_key,
                                         aname      => 'ORG_ID');

  fnd_client_info.set_org_context(to_char(l_org_id));

  if (display_type = 'text/html') then

    l_document := NL || NL || '<!-- REQ_DIST_DETAILS -->'|| NL || NL || '<P><B>';
    l_document := l_document || 'Line Distributions';
    l_document := l_document || '</B>';


    l_document := l_document || '<TABLE border=1 cellpadding=2 cellspacing=1>';

    l_document := l_document || '<TR>' || NL;

    l_document := l_document || '<TH>' || 'Line Number' || '</TH>' || NL;
    l_document := l_document || '<TH>' || 'Dist. Number' || '</TH>' || NL;
    l_document := l_document || '<TH>' || 'Distribution Account' || '</TH>' || NL;
    l_document := l_document || '<TH>' || 'Alloc. Type' || '</TH>' || NL;
    l_document := l_document || '<TH>' || 'Allocation' || '</TH>' || NL;

    l_document := l_document || '</TR>' || NL;

    open dist_csr(l_document_id);

    loop

      fetch dist_csr into l_line;

      exit when dist_csr%notfound;

      l_document := l_document || '<TR>' || NL;

      l_document := l_document || '<TD nowrap>' || nvl(to_char(l_line.line_num), '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD nowrap>' || nvl(to_char(l_line.dist_num), '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD nowrap>' || nvl(l_line.dist_acct, '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD nowrap>' || nvl(l_line.allocation_type, '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD nowrap>' || nvl(to_char(l_line.allocation_value), '&nbsp') || '</TD>' || NL;

      l_document := l_document || '</TR>' || NL;

    end loop;

    close dist_csr;

    l_document := l_document || '</TABLE></P>' || NL;

  else

    l_document := l_document || 'Line Distributions' || NL || NL;

    open dist_csr(l_document_id);

    loop

      fetch dist_csr into l_line;

      exit when dist_csr%notfound;

      l_document := l_document || '         Line Number:' || to_char(l_line.line_num)  || NL;
      l_document := l_document || '        Dist. Number:' || to_char(l_line.dist_num) || NL;
      l_document := l_document || 'Distribution Account:' || l_line.dist_acct || NL;
      l_document := l_document || '     Allocation Type:' || l_line.allocation_type || NL;
      l_document := l_document || '          Allocation:' || to_char(l_line.allocation_value) || NL;

      end loop;

      close dist_csr;

  end if;

  document := l_document;

END get_req_distr_details;



PROCEDURE get_req_attachments  (document_id     in      varchar2,
                                display_type    in      varchar2,
                                document        in out  varchar2,
                                document_type   in out  varchar2) IS

-- -------------------------------------------------------------------------
--  This procedure builds an HTML table containing a requisitions attachments
--  information for display in the Workflow Notification Message.
--
-- --------------------------------------------------------------------------

  v_do_header        varchar2(1) := 'Y';
  l_item_type        wf_items.item_type%TYPE;
  l_item_key         wf_items.item_key%TYPE;
  l_document_id      po_requisition_lines.requisition_header_id%TYPE;
  l_org_id           po_requisition_lines.org_id%TYPE;
  l_line             attachment_record;
  l_document         VARCHAR2(32000) := '';
  l_currency_code    fnd_currencies.currency_code%TYPE;
  NL                 VARCHAR2(1) := fnd_global.newline;


CURSOR attachment_csr(v_document_id NUMBER) IS
select fad.SEQ_NUM,
       fdctl.USER_NAME attention_to,
       fdd.USER_NAME doc_type,
       fdtl.DESCRIPTION,
       decode(fdd.USER_NAME,
              'Web Page',fdtl.FILE_NAME,
              'File', PORTAL_WF_REQ_AP_NOTIFICATION.get_file_attachment(fdtl.MEDIA_ID) || ' - Please log into the Oracle Applications to download this file',
              'Long Text',PORTAL_WF_REQ_AP_NOTIFICATION.get_long_text_attachment(fdtl.MEDIA_ID),
              '-- Attachment Type Not Recognized --'
             ) attachment_txt,
       decode(fdd.USER_NAME,
              'Web Page',fdtl.FILE_NAME,
              'File', PORTAL_WF_REQ_AP_NOTIFICATION.get_file_attachment(fdtl.MEDIA_ID) || '<BR> - Please log into the Oracle Applications to download this file' ||
                     '<BR><A HREF="' ||
                     PORTAL_WF_REQ_AP_NOTIFICATION.get_site_profile_option('PORTAL_REQ_APPROVAL_LOGIN_URL') ||
                     '">Log on to Oracle Applications</A>',
              'Long Text',replace(replace(PORTAL_WF_REQ_AP_NOTIFICATION.get_long_text_attachment(fdtl.MEDIA_ID),
                                          chr(10),'<BR>'), 
                                  chr(13),''),        
              '-- Attachment Type Not Recognized --'
             ) attachment_html      
  from FND_DOCUMENTS_TL fdtl,
       FND_DOCUMENT_CATEGORIES_TL fdctl,
       FND_DOCUMENT_DATATYPES fdd,
       FND_DOCUMENTS fd,
       FND_ATTACHED_DOCUMENTS fad,
       PO_REQUISITION_HEADERS prha
 where fdtl.LANGUAGE = USERENV('LANG')
   and fd.DOCUMENT_ID = fdtl.DOCUMENT_ID
   and fdctl.LANGUAGE = USERENV('LANG')
   and fd.CATEGORY_ID = fdctl.CATEGORY_ID
   and fdd.LANGUAGE = USERENV('LANG')
   and fd.DATATYPE_ID = fdd.DATATYPE_ID
   and fad.DOCUMENT_ID = fd.DOCUMENT_ID
   and fad.ENTITY_NAME = 'REQ_HEADERS'
   and prha.REQUISITION_HEADER_ID = fad.PK1_VALUE
   and prha.REQUISITION_HEADER_ID = v_document_id
 order by 1
 ;

BEGIN

  l_item_type := substr(document_id, 1, instr(document_id, ':') - 1);
  l_item_key := substr(document_id, instr(document_id, ':') + 1,
                       length(document_id) - 2);

  l_document_id := wf_engine.GetItemAttrNumber
                                        (itemtype   => l_item_type,
                                         itemkey    => l_item_key,
                                         aname      => 'DOCUMENT_ID');

  l_org_id := wf_engine.GetItemAttrNumber
                                        (itemtype   => l_item_type,
                                         itemkey    => l_item_key,
                                         aname      => 'ORG_ID');

  fnd_client_info.set_org_context(to_char(l_org_id));

  if (display_type = 'text/html') then
    
    open attachment_csr(l_document_id);

    loop

      fetch attachment_csr into l_line;

      exit when attachment_csr%notfound;
      
      if (v_do_header = 'Y') then
        v_do_header := 'N';
        l_document := NL || NL || '<!-- REQ_ATTACHMENT_DETAILS -->'|| NL || NL || '<P><B>';
        l_document := l_document || 'Requisition Attachments';
        l_document := l_document || '</B>';
        l_document := l_document || '<TABLE border=1 cellpadding=2 cellspacing=1>';
        l_document := l_document || '<TR>' || NL;
        l_document := l_document || '<TH>' || 'Seq Number' || '</TH>' || NL;
        l_document := l_document || '<TH>' || 'Attention To' || '</TH>' || NL;
        l_document := l_document || '<TH>' || 'Attachment Type' || '</TH>' || NL;
        l_document := l_document || '<TH>' || 'Description' || '</TH>' || NL;
        l_document := l_document || '<TH>' || 'Attachment' || '</TH>' || NL;
        l_document := l_document || '</TR>' || NL;
      end if;

      l_document := l_document || '<TR>' || NL;
      l_document := l_document || '<TD valign=top nowrap>' || nvl(to_char(l_line.seq_num), '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD valign=top nowrap>' || nvl(l_line.attention_to, '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD valign=top nowrap>' || nvl(l_line.doc_type, '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD valign=top>' || nvl(l_line.description, '&nbsp') || '</TD>' || NL;
      l_document := l_document || '<TD valign=top>' || nvl(l_line.attachment_html, '&nbsp') || '</TD>' || NL;
      l_document := l_document || '</TR>' || NL;

    end loop;

    close attachment_csr;

    if (v_do_header = 'N') then
      l_document := l_document || '</TABLE></P>' || NL;
    end if;  

  else


    open attachment_csr(l_document_id);

    loop

      fetch attachment_csr into l_line;

      exit when attachment_csr%notfound;

      if (v_do_header = 'Y') then
        v_do_header := 'N';
        l_document := l_document || 'Requisition Attachments' || NL || NL;
      end if;
        
      l_document := l_document || '     Seq Number:' || to_char(l_line.seq_num)  || NL;
      l_document := l_document || '   Attention To:' || l_line.attention_to || NL;
      l_document := l_document || 'Attachment Type:' || l_line.doc_type || NL;
      l_document := l_document || '    Description:' || l_line.description || NL;
      l_document := l_document || '     Attachment:' || l_line.attachment_txt || NL;

      end loop;

      close attachment_csr;

  end if;

  document := l_document;

END get_req_attachments;




END PORTAL_WF_REQ_AP_NOTIFICATION;
/

set define on


